/*
 * Counting the number of rotation of motors based on encoder signals.
 *
 * Author: BLADOWSKI Jakub
 * S4 Project 63: Follower Robot
 * Telecom Bretagne
 * February - June 2015
 */

#include "encoders.h"

/*
 * Definitions of pin numbers.
 */
#define M1_ENCODER_A 23
#define M1_ENCODER_B 25
#define M2_ENCODER_A 27
#define M2_ENCODER_B 29
#define M3_ENCODER_A 31
#define M3_ENCODER_B 33

/*
 * Encoder counter variables.
 */
volatile int encode_M1, encode_M2, encode_M3;

/*
 * Initialize input pins from encoders, register interrupts, reset counters.
 */
void setup_encoding() {
  pinMode(M1_ENCODER_A, INPUT);
  digitalWrite(M1_ENCODER_A, HIGH);
  attachInterrupt(M1_ENCODER_A, encoder_interrupt_M1, RISING);
  pinMode(M2_ENCODER_A, INPUT);
  digitalWrite(M2_ENCODER_A, HIGH);
  attachInterrupt(M2_ENCODER_A, encoder_interrupt_M2, RISING);
  pinMode(M3_ENCODER_A, INPUT);
  digitalWrite(M3_ENCODER_A, HIGH);
  attachInterrupt(M3_ENCODER_A, encoder_interrupt_M3, RISING);
  pinMode(M1_ENCODER_B, INPUT);
  pinMode(M2_ENCODER_B, INPUT);
  pinMode(M3_ENCODER_B, INPUT);
  reset_encoding();
}

/*
 * Set all encoder counters to 0.
 */
void reset_encoding() {
  encode_M1 = 0;
  encode_M2 = 0;
  encode_M3 = 0;
}

/*
 * Interrupt handlers for the three encoders.
 * Called on rising edge of encoder signal A.
 * Depending on value of signal B, increment or decrement the corresponding encoder counter.
 */
void encoder_interrupt_M1() {
  if (digitalRead(M1_ENCODER_B)) 
    encode_M1 += 1;
  else
    encode_M1 -= 1;
}
void encoder_interrupt_M2() {
  if (digitalRead(M2_ENCODER_B)) 
    encode_M2 += 1;
  else
    encode_M2 -= 1;
}
void encoder_interrupt_M3() {
  if (digitalRead(M3_ENCODER_B)) 
    encode_M3 += 1;
  else
    encode_M3 -= 1;
}

/*
 * Return current encoder counter value for given motor.
 */
int get_encoding(int motor) {
  if (motor == 0)
    return encode_M1;
  if (motor == 1)
    return encode_M2;
  if (motor == 2)
    return encode_M3;
}
