/*
 * Low-level control of motor rotation.
 *
 * Author: BLADOWSKI Jakub
 * S4 Project 63: Follower Robot
 * Telecom Bretagne
 * February - June 2015
 */

#include "motors.h"

/*
 * Definitions of pin numbers.
 */
#define M1_PWM_A 2
#define M1_PWM_B 3
#define M1_ENABLE_A 22
#define M1_ENABLE_B 24
  
#define M2_PWM_A 4
#define M2_PWM_B 5
#define M2_ENABLE_A 26
#define M2_ENABLE_B 28
  
#define M3_PWM_A 6
#define M3_PWM_B 7
#define M3_ENABLE_A 30
#define M3_ENABLE_B 32

/*
 * Initialize pins used for controlling motors.
 */
void setup_motors() {
  // MOTOR 1
  pinMode(M1_PWM_A, OUTPUT);
  pinMode(M1_PWM_B, OUTPUT);
  pinMode(M1_ENABLE_A, OUTPUT);
  pinMode(M1_ENABLE_B, OUTPUT);
  analogWrite(M1_PWM_A, 0);
  analogWrite(M1_PWM_B, 0);
  digitalWrite(M1_ENABLE_A, 0);
  digitalWrite(M1_ENABLE_B, 0);
  
  // MOTOR 2
  pinMode(M2_PWM_A, OUTPUT);
  pinMode(M2_PWM_B, OUTPUT);
  pinMode(M2_ENABLE_A, OUTPUT);
  pinMode(M2_ENABLE_B, OUTPUT);
  analogWrite(M2_PWM_A, 0);
  analogWrite(M2_PWM_B, 0);
  digitalWrite(M2_ENABLE_A, 0);
  digitalWrite(M2_ENABLE_B, 0);
  
  // MOTOR 3
  pinMode(M3_PWM_A, OUTPUT);
  pinMode(M3_PWM_B, OUTPUT);
  pinMode(M3_ENABLE_A, OUTPUT);
  pinMode(M3_ENABLE_B, OUTPUT);
  analogWrite(M3_PWM_A, 0);
  analogWrite(M3_PWM_B, 0);
  digitalWrite(M3_ENABLE_A, 0);
  digitalWrite(M3_ENABLE_B, 0);
}

/*
 * Set current PWM value of a given motor to given value.
 */
int set_motor_speed(int motor, float motor_speed) {
  int ENABLE_A, PWM_A, ENABLE_B, PWM_B;
  if (motor == MOTOR_1) {
    ENABLE_A = M1_ENABLE_A;
    PWM_A = M1_PWM_A;
    ENABLE_B = M1_ENABLE_B;
    PWM_B = M1_PWM_B;
  }
  if (motor == MOTOR_2) {
    ENABLE_A = M2_ENABLE_A;
    PWM_A = M2_PWM_A;
    ENABLE_B = M2_ENABLE_B;
    PWM_B = M2_PWM_B;
  }
  if (motor == MOTOR_3) {
    ENABLE_A = M3_ENABLE_A;
    PWM_A = M3_PWM_A;
    ENABLE_B = M3_ENABLE_B;
    PWM_B = M3_PWM_B;
  }
  
  if(motor_speed > 0)
  {
    byte byte_speed = (byte)(motor_speed * 255);
    digitalWrite(ENABLE_A, HIGH);
    analogWrite(PWM_A, 0);
    digitalWrite(ENABLE_B, LOW);
    analogWrite(PWM_B, byte_speed);
  }
  else if(motor_speed < 0)
  {
    byte byte_speed = (byte)((-motor_speed) * 255);
    digitalWrite(ENABLE_A, LOW);
    analogWrite(PWM_A, byte_speed);
    digitalWrite(ENABLE_B, HIGH);
    analogWrite(PWM_B, 0);
  }
  else
  {
    digitalWrite(ENABLE_A, HIGH);
    analogWrite(PWM_A, 0);
    digitalWrite(ENABLE_B, HIGH);
    analogWrite(PWM_B, 0);
  }
}
