/* Author: BLADOWSKI Jakub
 * S4 Project 63: Follower Robot
 * Telecom Bretagne
 * February - June 2015
 */
 
#define USE_USBCON
#include <ros.h>
#include <geometry_msgs/Vector3.h>
#include <std_msgs/String.h>
#include <Arduino.h>
#include "pid.h"
#include <math.h>

ros::NodeHandle  nh;

std_msgs::String str_msg;
ros::Publisher chatter("chatter", &str_msg);

int BAUD = 115200;

//coefficient converting the distance value from dist_msg to meter
float y_coeff = 10.0;
float x_coeff = 1.0;

//Position of the zero from the robot for the distance (in meter)
float y_offset = 1.0;

//Range of distance where the robot won't move (in meter)
float y_max = 2.0;
float y_min = 1.0;

//Range of angle where the robot won't move (in deg)
float angle_max = 5;
float angle_min = -5;

void movement_cmd( const geometry_msgs::Vector3& dist_msg){

  float y = -y_coeff * dist_msg.y + y_offset;
  float x = x_coeff * dist_msg.x/2;
  float angle = (tan(x/y)* 4068) / 71;
  
  if(/*angle < angle_min || angle > angle_max */ true)
  {
      //command_rotation(angle);
      String message = "y = " + String(y) + ", x = "+ String(x) + ", Rotation of " + String(angle) + " deg" + " (" + String(tan(x/y)) + " rad)";
      char rot[52];
      message.toCharArray(rot, 52);
      str_msg.data = rot;
      chatter.publish( &str_msg );
  }
  if(y < y_min || y > y_max )
    {
      if (y < y_min)
        {
          command_displacement((y_max + y_min)/2 - y, 0);
        }
      else
      {
        command_displacement(y - (y_max + y_min)/2, 180);
      }
      //String message = "Displacement of = " + String((y_max + y_min)/2 - y) + " meter";
      String message = "y position from kinect= " + String(y) + " meter";
      char dist[42];
      message.toCharArray(dist, 42);
      str_msg.data = dist;
      chatter.publish( &str_msg );
    } 
}

ros::Subscriber<geometry_msgs::Vector3> sub("torsoPosition", movement_cmd);

/*
 * Initialize elements at the start of the program.
 */
void setup() {
  //setup_serial();
  nh.getHardware()->setBaud(BAUD);
  nh.initNode();
  nh.advertise(chatter);
  nh.subscribe(sub);
  
  setup_motors();
  setup_encoding();
  setup_pid();
}

/*
 * Main loop of the program.
 * Launch scheduled actions if conditions are met.
 */
void loop() {

  nh.spinOnce();
  
  if (condition_stop_displacement())
    stop_displacement();
  
  if (condition_stop_rotation())
    stop_rotation();

  if (condition_pid_tic())
    pid_tic();
    
}

/*
 * Initialize communications via serial port.
 */
void setup_serial() {
  Serial.begin(115200);
  Serial.flush();
  Serial.setTimeout(2);
}

/*
 * Interpret commands at reception via serial port.
 */
/*void serialEvent() {
  char first = Serial.read();
  
  // displacement
  if (first == 'D') {
    float distance_m = Serial.parseFloat();
    float angle = Serial.parseFloat();
    command_displacement(distance_m, angle);
    Serial.println("D");
  }
  
  // rotation
  if (first == 'R') {
    float angle = Serial.parseFloat();
    command_rotation(angle);
    Serial.println("R");
  }
  
  // halt
  if (first == 'H') {
    command_halt();
    Serial.println("H");
  }
  
  // test - one motor with given speed
  if (first == 'T') {
    float motor = Serial.parseInt() - 1;
    float mspeed;
    if (Serial.peek() == '+') 
      mspeed = 0.1;
    else if (Serial.peek() == '-')
      mspeed = -0.1;
    else
      mspeed = Serial.parseFloat();
    test_run(motor, mspeed);
  }
  
  // re-enable PID
  if (first == 'Z') {
    setup_pid();
  }
 
}*/

/*
 * Run a given motor at a given speed for a second without PID control following a test command.
 */
void test_run(int motor, float mspeed) {
  disable_pid();
  reset_encoding();
  set_motor_speed(motor, mspeed);
  delay(1000);
  Serial.println("test 1s");
  Serial.println(get_encoding(motor));
  set_motor_speed(motor, 0);
}
